package
{
	import away3d.animators.ParticleAnimationSet;
	import away3d.animators.ParticleAnimator;
	import away3d.animators.data.ParticleProperties;
	import away3d.animators.data.ParticlePropertiesMode;
	import away3d.animators.nodes.ParticlePositionNode;
	import away3d.containers.View3D;
	import away3d.core.base.Geometry;
	import away3d.debug.AwayStats;
	import away3d.entities.Mesh;
	import away3d.materials.TextureMaterial;
	import away3d.primitives.CubeGeometry;
	import away3d.textures.BitmapTexture;
	import away3d.tools.helpers.ParticleGeometryHelper;
	import away3d.utils.Cast;

	import flash.display.StageAlign;
	import flash.display.StageScaleMode;
	import flash.events.Event;
	import flash.geom.Vector3D;

	[SWF(frameRate = "60", width = "960", height = "540")]
	/**
	 * パーティクルバージョン (Away3D 4.1 alpha を利用)
	 * @author Yasunobu Ikeda
	 */
	public class ParticleCubes extends View3D
	{

		//----------------------------------------------------------
		//
		//   Static Property 
		//
		//----------------------------------------------------------

		/** 一辺あたりのオブジェクトの数。実行結果は CELL_NUM × CELL_NUM × CELL_NUM 個のオブジェクトが配置されます。 */
		private static const CELL_NUM:uint = 35;

		//----------------------------------------------------------
		//
		//   Constructor 
		//
		//----------------------------------------------------------

		public function ParticleCubes()
		{
			stage.scaleMode = StageScaleMode.NO_SCALE;
			stage.align = StageAlign.TOP_LEFT;
			antiAlias = 0;

			addChild(new AwayStats(this));

			// パーティクルのジオメトリを作成
			var cube:Geometry = new CubeGeometry(15, 15, 15);
			var geometrySet:Vector.<Geometry> = new Vector.<Geometry>();
			for (var i:int = 0; i < CELL_NUM * CELL_NUM * CELL_NUM; i++)
				geometrySet.push(cube);

			// パーティクルのアニメーションセットを作成
			var animationSet:ParticleAnimationSet = new ParticleAnimationSet();
			animationSet.addAnimation(new ParticlePositionNode(ParticlePropertiesMode.LOCAL_STATIC));
			animationSet.initParticleFunc = initParticleFunc;

			// パーティクル用のマテリアルを作成
			var texture:BitmapTexture = Cast.bitmapTexture(new ImageCls().bitmapData);
			var material:TextureMaterial = new TextureMaterial(texture);

			// パーティクルのメッシュを作成
			var animator:ParticleAnimator = new ParticleAnimator(animationSet);
			mesh = new Mesh(ParticleGeometryHelper.generateGeometry(geometrySet), material);
			mesh.animator = animator; // アニメーターを適用
			scene.addChild(mesh);

			// パーティクルアニメーターを開始
			animator.start();

			// エンターフレーム
			addEventListener(Event.ENTER_FRAME, enterFrameHandler);
		}

		//----------------------------------------------------------
		//
		//   Property 
		//
		//----------------------------------------------------------

		[Embed(source = "/assets/image.png")]
		private var ImageCls:Class;
		private var countForParticle:int = 0;
		private var mesh:Mesh;

		//----------------------------------------------------------
		//
		//   Function 
		//
		//----------------------------------------------------------

		/** パーティクルのプロパティを設定する関数 */
		private function initParticleFunc(prop:ParticleProperties):void
		{
			var indexK:int = countForParticle % CELL_NUM;
			var count2:int = Math.floor(countForParticle / CELL_NUM);
			var indexJ:int = count2 % CELL_NUM;
			var indexI:int = Math.floor(count2 / CELL_NUM);

			prop[ParticlePositionNode.POSITION_VECTOR3D] = new Vector3D(
				25 * (indexI - CELL_NUM / 2),
				25 * (indexJ - CELL_NUM / 2),
				25 * (indexK - CELL_NUM / 2)
				);

			countForParticle++;
		}

		/** レンダリングループ */
		private function enterFrameHandler(event:Event):void
		{
			mesh.rotationX += 1;
			mesh.rotationY += 1;

			render();
		}
	}
}
